@@ -1,4 +1,4 @@
-import pickle, os.path, fcntl, time, shutil, logging, platform, xmlrpclib, sys
+import pickle, os.path, fcntl, time, shutil, logging, platform, xmlrpclib, sys, itertools
 import role, package, util, error
 try:
     from hashlib import sha1 as sha
@@ -58,6 +58,14 @@ class Local(object):
         [mixins.discard(role) for role in self.roles]
         return list(mixins)
 
+    @property
+    def ordered_roles(self):
+        '''Return the list of roles, including mixins, in installation order (deps first)'''
+        self.mark_role_order()
+        ordered = self.roles + self.mixed_roles
+        ordered.sort(key=lambda x: -self.role(x)._order)
+        return ordered
+    
     def load(self):
         '''Load the saved host data from the serialized snapshot file'''
         save_path = os.path.join(self.path, '.save')
@@ -126,6 +134,19 @@ class Local(object):
             self._roles[name] = role.Role(self, name)
 
         return self._roles[name]
+    
+    def mark_role_order(self):
+        '''Go through all roles and mark the order field based on the role's installation depth'''
+        def recurse_order(roles, order=1):
+            if len(roles) == 0:
+                return
+
+            for name in reversed(roles):
+                self.role(name)._order = order
+                order += 1
+            recurse_order(list(itertools.chain(*[self.role(name).mixins for name in roles])), order)
+
+        recurse_order(self.roles)
 
     def sync_down(self):
         '''Copy from the repository to the current working directory'''
@@ -337,7 +358,7 @@ class Local(object):
                 else:
                     prepackages[script_path] = [dep]
 
-        for script_path, packages in prepackages.items():
+        for script_path, packages in self.script_order(prepackages):
             os.chmod(script_path, 0755)
             shell_args = [script_path]
             shell_args.extend(packages)
@@ -387,7 +408,7 @@ class Local(object):
 
     def run_preinstalls(self, changes):
         '''Scan the changed files looking for preinstall scripts and run them.  A preinstall script failure should terminate further installation.'''
-        for script, files in self.pivot_meta(changes, 'preinstall').iteritems():
+        for script, files in self.script_order(self.pivot_meta(changes, 'preinstall')):
             os.chmod(script, 0755)
             shell_args = [script]
             shell_args.extend(['/%s' % name for name in files])
@@ -404,7 +425,7 @@ class Local(object):
             if os.path.exists(script_path) and script_path not in postinstalls and os.path.getmtime(script_path) > self.last_install:
                 postinstalls[script_path] = []
 
-        for script, files in postinstalls.iteritems():
+        for script, files in self.script_order(postinstalls):
             os.chmod(script, 0755)
             shell_args = [script]
             shell_args.extend(['/%s' % name for name in files])
@@ -428,6 +449,18 @@ class Local(object):
 
         return meta
 
+    def script_order(self, scripts):
+        '''Take a dictionary of scripts: args and return an iterator for them in depth-first sorted order'''
+        ordered = []
+        for script, args in scripts.iteritems():
+            ordered.append((script, args))
+
+        if len(ordered) > 0:
+            self.mark_role_order()
+            ordered.sort(key=lambda x: -self.role(os.path.basename(os.path.dirname(x[0])))._order)
+
+        return ordered
+    
     def get_changes(self, install_root='/'):
         '''Scan the build directories for files that are different than what's in the install root, using sha1 hashes to compare file contents'''
         changed = []
